#include <QCoreApplication>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <QCommandLineParser>
#include "autologin.h"

int init_daemon()
{
    int  fd;

    switch (fork()) {
    case -1:
        return -1;
    case 0:
        break;
    default:
        exit(0);
    }

    if (setsid() == -1) {
        return -1;
    }

    umask(0);
    chdir ("/");

    fd = open("/dev/null", O_RDWR);
    if (fd == -1) {
        return -1;
    }
    if (dup2(fd, STDIN_FILENO) == -1) {
        return -1;
    }

    if (dup2(fd, STDOUT_FILENO) == -1) {
        return -1;
    }

     if (fd > STDERR_FILENO) {
        if (close(fd) == -1) {
            return -1;
        }
    }
    return 0;
}


int main(int argc, char *argv[])
{
    init_daemon();

    QCoreApplication app(argc, argv);
    QCommandLineParser parser;

    QCommandLineOption userNameOpt(QStringLiteral("username"));
    userNameOpt.setValueName(QStringLiteral("User's personnel account"));
    parser.addOption(userNameOpt);

    QCommandLineOption passwordOpt(QStringLiteral("password"));
    passwordOpt.setValueName(QStringLiteral("User's personnel password"));
    parser.addOption(passwordOpt);

    QCommandLineOption token(QStringLiteral("token"));
    token.setValueName(QStringLiteral("authenticate   token"));
    parser.addOption(token);
    parser.process(app);

    QString username = parser.value(userNameOpt);
    QString password = parser.value(passwordOpt);
    QString tokenCode = parser.value(token);
    AutoLogin autologin(username, password, tokenCode);
    QObject::connect(&autologin, &AutoLogin::exit, &app, QCoreApplication::quit);
    return app.exec();
}
