/***************************************************************************
* Copyright (c) 2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
* Copyright (c) 2013 Abdurrahman AVCI <abdurrahmanavci@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the
* Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
***************************************************************************/

#include "GreeterProxy.h"

#include "Configuration.h"
#include "Messages.h"
#include "SessionModel.h"
#include "SocketWriter.h"

#include <QLocalSocket>
#include <QString>
#include <QDBusReply>
#include <QDBusInterface>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "Configuration.h"

//socket客户端，将登录用户名和密码发送发送给sddm-udstokenserver服务端
int UdsSocketClient(char *username, char *password)
{
    char snd_name[1024];
    char snd_pass[1024];

    struct sockaddr_in addr;
    addr.sin_family=AF_INET;
    addr.sin_port=htons(5188);
    addr.sin_addr.s_addr=inet_addr("127.0.0.1");
    int sockid=socket(AF_INET,SOCK_STREAM,0);
    socklen_t addrlen=sizeof(addr);
    if(connect(sockid,(struct sockaddr *)&addr,addrlen)<0)
    {
        printf("connect\n");
        return 0;
    }
    memset(snd_name, 0, 1024);
    strcpy(snd_name, username);
    memset(snd_pass, 0, 1024);
    strcpy(snd_pass, password);
    write(sockid,snd_name,sizeof(snd_name));
    write(sockid,snd_pass,sizeof(snd_pass));

    close(sockid);
    return 0;
}

namespace SDDM {
    class GreeterProxyPrivate {
    public:
        SessionModel *sessionModel { nullptr };
        QLocalSocket *socket { nullptr };
        com::vm::daemon *daemonInterface { nullptr };
        QTimer *autoLoginTimer;
        QString hostName;
        QString username;
        QString password;
        int loginCount { 0 };
        int tryCount { 0 };
        bool canPowerOff { false };
        bool canReboot { false };
        bool canSuspend { false };
        bool canHibernate { false };
        bool canHybridSleep { false };
        bool isVirtualMachine { false };

    };

    GreeterProxy::GreeterProxy(const QString &socket, QObject *parent) : QObject(parent), d(new GreeterProxyPrivate()) {
        d->socket = new QLocalSocket(this);
        // connect signals
        connect(d->socket, SIGNAL(connected()), this, SLOT(connected()));
        connect(d->socket, SIGNAL(disconnected()), this, SLOT(disconnected()));
        connect(d->socket, SIGNAL(readyRead()), this, SLOT(readyRead()));
        connect(d->socket, SIGNAL(error(QLocalSocket::LocalSocketError)), this, SLOT(error()));

        // connect to server
        d->socket->connectToServer(socket);

        d->daemonInterface = new com::vm::daemon("com.vm.daemon",
                                                       "/daemon",QDBusConnection::systemBus(), this);
        d->daemonInterface->initStatus();
        connect(d->daemonInterface, &com::vm::daemon::autoLogin, this, &GreeterProxy::loginDomain);
        connect(d->daemonInterface, &com::vm::daemon::tokenSuccess, this, [=]{
            login(d->daemonInterface->userName(), QString(), d->sessionModel->lastIndex(), true);
        });
        d->autoLoginTimer = new QTimer (this);
        d->autoLoginTimer->setInterval(1000);
        connect(d->autoLoginTimer, &QTimer::timeout, this, &GreeterProxy::loginDomain);

        //未加域，是中兴内部版本，可以连通zte.intra,弹出加域输入框
        if(!isAdded() && d->daemonInterface->isZte() && !d->daemonInterface->ping())
            d->autoLoginTimer->start();
    }

    GreeterProxy::~GreeterProxy() {
        delete d;
    }
    void GreeterProxy::loginDomain()
    {
        d->username = d->daemonInterface->userName();
        d->password = d->daemonInterface->password();
        if(!d->password.isEmpty() && !d->username.isEmpty())
        {
            if(!isAdded())
            {
                addDomain();
            }
            else
                login(d->username, d->password, d->sessionModel->lastIndex());
            d->autoLoginTimer->stop();
            return;
        }
        d->loginCount ++;
        if(d->loginCount > 10 &&  !isAdded())
        {
            emit autoLoginFailed();
            d->autoLoginTimer->stop();
            d->loginCount = 0;
        }
    }
    void GreeterProxy::setUser(const QString &user, const QString &password)
    {
        d->daemonInterface->setUser(user, password , "");
    }
    bool GreeterProxy::isAdded()
    {
        QProcess process;
        process.start(QStringLiteral("/usr/sbin/realm"), QStringList {} << "list");
        process.waitForFinished();
        QList<QByteArray> output =  process.readAllStandardOutput().split('\n');
        for(int i = 0 ; i < output.size() ; i++)
        {
            QByteArray line = output.at(i).trimmed();
            QList<QByteArray> userNameList = line.split(':');
            if(line.startsWith("permitted-logins") &&
                    !userNameList.at(1).trimmed().isEmpty())
            {
                return true;
            }
        }
        return false;
    }
    void GreeterProxy::addDomain()
    {
        emit loggingIn();
        d->daemonInterface->start();
        d->tryCount++;
        connect(d->daemonInterface, &com::vm::daemon::finished, this, [=](int success, const QString &message){
            if(!success)
                login(d->username, d->password, d->sessionModel->lastIndex());
            else
            {
                emit tryAgain(d->tryCount);
            }
        });
    }
    const QString &GreeterProxy::hostName() const {
        return d->hostName;
    }

    void GreeterProxy::setSessionModel(SessionModel *model) {
        d->sessionModel = model;
    }

    bool GreeterProxy::canPowerOff() const {
        return d->canPowerOff;
    }

    bool GreeterProxy::canReboot() const {
        return d->canReboot;
    }

    bool GreeterProxy::canSuspend() const {
        return d->canSuspend;
    }

    bool GreeterProxy::canHibernate() const {
        return d->canHibernate;
    }

    bool GreeterProxy::canHybridSleep() const {
        return d->canHybridSleep;
    }

    bool GreeterProxy::isVirtualMachine() const {
        return d->isVirtualMachine;
    }

    bool GreeterProxy::isConnected() const {
        return d->socket->state() == QLocalSocket::ConnectedState;
    }

    void GreeterProxy::powerOff() {
        SocketWriter(d->socket) << quint32(GreeterMessages::PowerOff);
    }

    void GreeterProxy::reboot() {
        SocketWriter(d->socket) << quint32(GreeterMessages::Reboot);
    }

    void GreeterProxy::suspend() {
        SocketWriter(d->socket) << quint32(GreeterMessages::Suspend);
    }

    void GreeterProxy::hibernate() {
        SocketWriter(d->socket) << quint32(GreeterMessages::Hibernate);
    }

    void GreeterProxy::hybridSleep() {
        SocketWriter(d->socket) << quint32(GreeterMessages::HybridSleep);
    }

    void GreeterProxy::login(const QString &user, const QString &password, const int sessionIndex, bool isAutoLogin) const {
        if (!d->sessionModel) {
            // log error
            qCritical() << "Session model is not set.";

            // return
            return;
        }

        // get model index
        QModelIndex index = d->sessionModel->index(sessionIndex, 0);

        // send command to the daemon
        Session::Type type = static_cast<Session::Type>(d->sessionModel->data(index, SessionModel::TypeRole).toInt());
        QString name = d->sessionModel->data(index, SessionModel::FileRole).toString();
        Session session(type, name);
        SocketWriter(d->socket) << quint32(GreeterMessages::Login) << user << password << session << isAutoLogin;
        d->username = user;
        d->password = password;
    }

    void GreeterProxy::connected() {
        // log connection
        qDebug() << "Connected to the daemon.";

        // send connected message
        SocketWriter(d->socket) << quint32(GreeterMessages::Connect);
    }

    void GreeterProxy::disconnected() {
        // log disconnection
        qDebug() << "Disconnected from the daemon.";
    }

    void GreeterProxy::error() {
        qCritical() << "Socket error: " << d->socket->errorString();
    }

    bool GreeterProxy::checkVirtualMachine(){
        QProcess p;
        QStringList options;

        options << "-c" << "grep -q \"^flags.*\\ hypervisor\\ \" /proc/cpuinfo";
        p.start("/bin/bash", options);
        p.waitForFinished();
        p.waitForReadyRead();

        int exitCode = p.exitCode();
        return exitCode ? false : true;
    }

    void GreeterProxy::readyRead() {
        // input stream
        QDataStream input(d->socket);

        while (input.device()->bytesAvailable()) {
            // read message
            quint32 message;
            input >> message;

            switch (DaemonMessages(message)) {
                case DaemonMessages::Capabilities: {
                    // log message
                    qDebug() << "Message received from daemon: Capabilities";

                    // read capabilities
                    quint32 capabilities;
                    input >> capabilities;

                    // parse capabilities
                    d->canPowerOff = capabilities & Capability::PowerOff;
                    d->canReboot = capabilities & Capability::Reboot;
                    d->canSuspend = capabilities & Capability::Suspend;
                    d->canHibernate = capabilities & Capability::Hibernate;
                    d->canHybridSleep = capabilities & Capability::HybridSleep;
                    d->isVirtualMachine = checkVirtualMachine();


                    // emit signals
                    emit canPowerOffChanged(d->canPowerOff);
                    emit canRebootChanged(d->canReboot);
                    emit canSuspendChanged(d->canSuspend);
                    emit canHibernateChanged(d->canHibernate);
                    emit canHybridSleepChanged(d->canHybridSleep);
                    emit isVirtualMachineChanged(d->isVirtualMachine);
                }
                break;
                case DaemonMessages::HostName: {
                    // log message
                    qDebug() << "Message received from daemon: HostName";

                    // read host name
                    input >> d->hostName;

                    // emit signal
                    emit hostNameChanged(d->hostName);
                }
                break;
				case DaemonMessages::LoggingIn: {
            		qDebug() << "Message received from daemon: LoggingIn";

            		emit loggingIn();
            	break;
        		}
                case DaemonMessages::LoginSucceeded: {
                    // log message
                    qDebug() << "Message received from daemon: LoginSucceeded";
                    //登录成功发送用户名及密码给单点登录服务端
                    UdsSocketClient(d->username.toLatin1().data(), d->password.toLatin1().data());
                    if(d->daemonInterface->isZte() && isAdded())
                        d->daemonInterface->deleteCommonUser();
                    // emit signal
                    emit loginSucceeded();
                }
                break;
                case DaemonMessages::LoginFailed: {
                    // log message
                    qDebug() << "Message received from daemon: LoginFailed";

                    // emit signal
                    emit loginFailed();
                }
                break;
                default: {
                    // log message
                    qWarning() << "Unknown message received from daemon.";
                }
            }
        }
    }
}
