#include "uacverify.h"
#include <QJsonObject>
#include <QJsonDocument>
#include <QNetworkReply>
#include <QMessageAuthenticationCode>
#include <QHostAddress>
#include <QNetworkInterface>

UACVerify::UACVerify(QObject *parent) : QObject(parent)
{
    accessManager = new  QNetworkAccessManager(this);
    connect(accessManager, &QNetworkAccessManager::finished, this, &UACVerify::finished);
}
void UACVerify::finished(QNetworkReply* reply)
{
    QByteArray replyStr = reply->readAll();
    QString enMsg;
    QJsonParseError jsonError;
    QJsonDocument doucment = QJsonDocument::fromJson(replyStr, &jsonError);
    if(!doucment.isNull() && jsonError.error == QJsonParseError::NoError)
    {
        if(doucment.isObject())
        {
            QJsonObject object = doucment.object();
            QString result;
            if(object.contains("code"))
            {
                QJsonValue value = object.value("code");
                if(value.isObject())
                {
                    QJsonObject arrayObject = value.toObject();
                    if(arrayObject.contains("code"))
                    {
                        result = arrayObject.value("code").toString();
                        if(result != "0000")
                        {
                            emit checkResult(false, arrayObject.value("msg").toString());
                            return;
                        }
                    }
                }
            }
            if(object.contains("bo"))
            {
                QJsonValue value = object.value("bo");
                if(value.isObject())
                {
                    QJsonObject arrayObject = value.toObject();
                    if(arrayObject.contains("code"))
                    {
                        QString code = arrayObject.value("code").toString();
                        if(result == "0000" && code == "0000")
                        {
                            emit checkResult(true, arrayObject.value("msg").toString());
                            return;
                        }
                        enMsg = arrayObject.value("enMsg").toString();
                    }
                }
            }
        }
    }
    emit checkResult(false, enMsg);
}
void UACVerify::checkUac(const QString &userName, const QString &token)
{
    QJsonObject json;
    QString systemCode = "UAC";
    //生产环境标识
    QByteArray environment = "prod";
    //申请的accesskey
    QByteArray accesskey = "nsdl0uy0krnrimrbs7mt8pp2cc1z4wt7";
    //申请的密钥accessSecret
    QByteArray accessSecret = "b77b3d858ee35f9b346639d3ede0ed3b94a1e87a6c4b91794bfcc5ca498e31b0";
    //验证url
    QUrl uacUrl("https://uac.zte.com.cn/uactoken/auth/token/verify.serv");
    QString LocalIp;
    QList<QHostAddress> ipList = QNetworkInterface::allAddresses();
    //获取内网ip
    for(QHostAddress ipItem : ipList)
    {
        if(ipItem.protocol() == QAbstractSocket::IPv4Protocol &&
                ipItem!=QHostAddress::Null &&
                ipItem!=QHostAddress::LocalHost &&
                ipItem.toString().left(2)=="10")
        {
            LocalIp = ipItem.toString();
        }
    }

    json.insert("account", userName);
    json.insert("token", token);
    json.insert("clientIP", LocalIp);
    json.insert("systemCode", systemCode);

    QString md5;
    QString sums = userName + token + LocalIp + systemCode;
    QByteArray bb;
    bb = QCryptographicHash::hash (sums.toLatin1(), QCryptographicHash::Md5 );
    md5.append(bb.toHex());
    json.insert("verifyCode", md5);
    QJsonDocument document;
    document.setObject(json);
    //json格式的post数据，参考令牌校验文档
    QByteArray data = document.toJson(QJsonDocument::Compact);
    //1970年到现在的毫秒时间戳
    qint64 timestamp = QDateTime::currentMSecsSinceEpoch();
    //使用HmacSHA256加密算法
    QByteArray sign = QMessageAuthenticationCode::hash(accesskey + environment + QString::number(timestamp).toUtf8(),
                                                       accessSecret, QCryptographicHash::Sha256).toHex();
    //http签名头
    QByteArray signature = accesskey + ":" + QString::number(timestamp).toUtf8() + ":" +sign;

    QNetworkRequest request;
    request.setUrl(uacUrl);
    request.setHeader(QNetworkRequest::ContentTypeHeader,QVariant("application/json"));
    request.setRawHeader("X-Emp-No", userName.toUtf8());
    request.setRawHeader("X-Signature", signature.toPercentEncoding());

    //post请求
    accessManager->post(request, data);
}
