#include "vmdaemon.h"
#include "daemonadaptor.h"
#include <QProcess>
#include <pwd.h>
#include <sys/types.h>
#include <sys/unistd.h>
#include <statgrab.h>
#include<sys/stat.h>
#include "uacverify.h"

VMDaemon::VMDaemon(QObject *parent) : QObject(parent),
    mProcess(new QProcess(this)),
    mCount(0),
    mUserName(""),
    mPassword(""),
    mUacVeriy(new UACVerify(this))
{
    new DaemonAdaptor (this);
    if (!QDBusConnection::systemBus().isConnected()) {
        qWarning() << "Cannot connect to the D-Bus session bus.";
    }

    if (!QDBusConnection::systemBus().registerService(QStringLiteral("com.vm.daemon"))) {
        qWarning() << QDBusConnection::systemBus().lastError().message();
    }
    QDBusConnection::systemBus().registerObject(QStringLiteral("/daemon"), QStringLiteral("com.vm.daemon"), this);
    connect(mUacVeriy, &UACVerify::checkResult, this, [=](bool success, QString msg){
        if(success)
        {
            mToken.clear();
            emit tokenSuccess();
        }
        else
        {
            qWarning() << "check Failed:" << msg;
        }
    });
}
void VMDaemon::start()
{
    connect(mProcess, QOverload<int>::of(&QProcess::finished), [=](int exitCode){
        if(!exitCode)
        {
            emit finished(exitCode, QString());

            mCount = 0;
        }
        else
        {
            mCount ++;
            if(mCount < 3)
            {
                startDomain();
            }
            else
            {
                //三次失败之后，退域，并且发送失败消息
                QString msg =  QString::fromUtf8(mProcess->readAllStandardError());
                qWarning() << "add domain Failed:" <<msg;
                emit finished(exitCode, msg);
                QProcess::execute("/usr/bin/realm", QStringList {} <<"leave");
            }
        }
    });

    startDomain();
}
void  VMDaemon::startDomain()
{
    mProcess->start("/usr/bin/add_domain_dm", QStringList{} << "-u" << mUserName);
}
bool VMDaemon::setUser(const QString &userName, const QString &password, const QString &token)
{

    int ret = true;
    if(!userName.contains("@zte"))
        return false;
    mUserName = userName;
    if(!password.isEmpty() && password.compare("nde-default") != 0)
    {
        mPassword = password;
    }
    if(!token.isEmpty())
        mToken = token;
    QTimer::singleShot(0, this, [=]{
        if(!isAdded())
            QProcess::execute("/usr/bin/set_domain",
                          QStringList {} << "-p" << mUserName);
    });
    QTimer::singleShot(1, this, [=]{
        if(isAdded() && !mUserName.isEmpty() && !mToken.isEmpty())
            mUacVeriy->checkUac(mUserName.split("@").first(), mToken);
    });
    if(!mUserName.isEmpty() && !mPassword.isEmpty())
        emit autoLogin();
    return ret;
}
QString VMDaemon::password()
{
    return mPassword;
}
QString VMDaemon::userName()
{
    return mUserName;
}
bool VMDaemon::isAdded()
{
    QProcess process;
    process.start(QStringLiteral("/usr/sbin/realm"), QStringList {} << "list");
    process.waitForFinished();
    QList<QByteArray> output =  process.readAllStandardOutput().split('\n');
    for(int i = 0 ; i < output.size() ; i++)
    {
        QByteArray line = output.at(i).trimmed();
        QList<QByteArray> userNameList = line.split(':');
        if(line.startsWith("permitted-logins") &&
                !userNameList.at(1).trimmed().isEmpty())
        {
            return true;
        }
    }
    return false;
}
void VMDaemon::deleteCommonUser()
{
    struct passwd *current_pw = nullptr;
    while ((current_pw = getpwent()) !=  nullptr) {
        if (int(current_pw->pw_uid) < 1000 || int(current_pw->pw_uid) > 60000)
            continue;
        QProcess process;
        process.start("/sbin/userdel", QStringList {} << current_pw->pw_name);
        process.waitForFinished();
    }
    endpwent();
}
bool VMDaemon::isZte()
{
    QString release = "/etc/os-release";
    QFile file(release);
    if(file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        while(!file.atEnd())

        {
            QByteArray line = file.readLine().trimmed();
            if(line.startsWith("VERSION") && line.endsWith("ZTE"))
            {
                file.close();
                return true;
            }
        }
    }
    file.close();
    return false;
}
void VMDaemon::initStatus()
{
    QString lockStatus = "/etc/lock-status.conf";
    QFile file(lockStatus);
    if(file.open(QIODevice::WriteOnly|QIODevice::Text))
    {
        QTextStream out(&file);
        out << "status=" << 0;
    }
    file.close();
    chown(lockStatus.toLocal8Bit().data(), 0, 0);
    chmod(lockStatus.toLocal8Bit().data(), 0777);
}

bool VMDaemon::ping()
{
    QProcess process;
    process.start("/usr/bin/ping", QStringList {} << "zte.intra" << "-c" << "1");
    process.waitForFinished();
    return process.exitCode();
}
